@@ -113,8 +113,10 @@ authreg_t authreg_init(c2s_t c2s, char *name) {
 
 /** shutdown the authreg system */
 void authreg_free(authreg_t ar) {
-    if(ar->free != NULL) (ar->free)(ar);
-    free(ar);
+    if (ar) {
+        if(ar->free != NULL) (ar->free)(ar);
+        free(ar);
+    }
 }
 
 /** auth get handler */
@@ -172,7 +174,7 @@ static void _authreg_auth_get(c2s_t c2s, sess_t sess, nad_t nad) {
     nad_free(nad);
 
     /* build a result packet */
-    nad = nad_new(sess->s->nad_cache);
+    nad = nad_new();
 
     ns = nad_add_namespace(nad, uri_CLIENT, NULL);
 
@@ -321,7 +323,7 @@ static void _authreg_auth_set(c2s_t c2s, sess_t sess, nad_t nad) {
     /* now, are they authenticated? */
     if(authd)
     {
-        log_write(c2s->log, LOG_NOTICE, "[%d] legacy authentication succeeded: host=%s, username=%s, resource=%s%s", sess->s->tag, sess->host->realm, username, resource, sess->s->ssf ? ", TLS negotiated" : "");
+        log_write(c2s->log, LOG_NOTICE, "[%d] legacy authentication succeeded: host=%s, username=%s, resource=%s%s%s", sess->s->tag, sess->host->realm, username, resource, sess->s->ssf ? ", TLS negotiated" : "", sess->s->compressed ? ", ZLIB compression enabled" : "");
 
         /* create new bound jid holder */
         if(sess->resources == NULL) {
@@ -338,7 +340,7 @@ static void _authreg_auth_set(c2s_t c2s, sess_t sess, nad_t nad) {
         log_write(sess->c2s->log, LOG_NOTICE, "[%d] requesting session: jid=%s", sess->s->tag, jid_full(sess->resources->jid));
 
         /* build a result packet, we'll send this back to the client after we have a session for them */
-        sess->result = nad_new(sess->s->nad_cache);
+        sess->result = nad_new();
 
         ns = nad_add_namespace(sess->result, uri_CLIENT, NULL);
 
@@ -385,7 +387,7 @@ static void _authreg_register_get(c2s_t c2s, sess_t sess, nad_t nad) {
     nad_free(nad);
 
     /* build a result packet */
-    nad = nad_new(sess->s->nad_cache);
+    nad = nad_new();
 
     ns = nad_add_namespace(nad, uri_CLIENT, NULL);
 
@@ -459,7 +461,7 @@ static void _authreg_register_set(c2s_t c2s, sess_t sess, nad_t nad)
         log_write(c2s->log, LOG_NOTICE, "[%d] registration remove succeeded, requesting user deletion: jid=%s", sess->s->tag, jid_user(sess->resources->jid));
 
         /* make a result nad */
-        sess->result = nad_new(sess->s->nad_cache);
+        sess->result = nad_new();
 
         ns = nad_add_namespace(sess->result, uri_CLIENT, NULL);
 
@@ -564,7 +566,7 @@ static void _authreg_register_set(c2s_t c2s, sess_t sess, nad_t nad)
     log_debug(ZONE, "updated auth creds for %s", username);
 
     /* make a result nad */
-    sess->result = nad_new(sess->s->nad_cache);
+    sess->result = nad_new();
 
     ns = nad_add_namespace(sess->result, uri_CLIENT, NULL);
 
