@@ -58,12 +58,6 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
                         sess->rate_log = 1;
                     }
 
-                    log_write(sess->c2s->log, LOG_NOTICE, "%d is throttled, disconnecting", sess->fd->fd);
-
-                    /* Disconnect the user.  Ideally we would just stop
-                       reading from their socket until the throttle time
-                       expires.  But that's difficult. */
-                    sx_kill(s);
                     return -1;
                 }
 
@@ -96,7 +90,7 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
                     log_write(sess->c2s->log, LOG_NOTICE, "[%d] [%s, port=%d] read error: %s (%d)", sess->fd->fd, sess->ip, sess->port, MIO_STRERROR(MIO_ERROR), MIO_ERROR);
 
                 sx_kill(s);
-                
+
                 return -1;
             }
 
@@ -122,18 +116,18 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
                     "Cache-control: private\r\n"
                     "Connection: close\r\n\r\n";
                 char *answer;
-    
+
                 len = strlen(sess->c2s->http_forward) + strlen(http);
                 answer = malloc(len * sizeof(char));
                 sprintf (answer, http, sess->c2s->http_forward);
-    
+
                 log_write(sess->c2s->log, LOG_NOTICE, "[%d] bouncing HTTP request to %s", sess->fd->fd, sess->c2s->http_forward);
-    
+
                 /* send HTTP answer */
                 len = send(sess->fd->fd, answer, len-1, 0);
 
                 free(answer);
-    
+
                 /* close connection */
                 sx_kill(s);
 
@@ -155,14 +149,14 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
 
             if(MIO_WOULDBLOCK)
                 return 0;
-            
+
             if(s->state >= state_OPEN && sess->resources != NULL)
                 log_write(sess->c2s->log, LOG_NOTICE, "[%d] [%s] write error: %s (%d)", sess->fd->fd, jid_user(sess->resources->jid), MIO_STRERROR(MIO_ERROR), MIO_ERROR);
             else
                 log_write(sess->c2s->log, LOG_NOTICE, "[%d] [%s. port=%d] write error: %s (%d)", sess->fd->fd, sess->ip, sess->port, MIO_STRERROR(MIO_ERROR), MIO_ERROR);
-        
+
             sx_kill(s);
-        
+
             return -1;
 
         case event_ERROR:
@@ -173,7 +167,7 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
                 log_write(sess->c2s->log, LOG_NOTICE, "[%d] [%s, port=%d] error: %s (%s)", sess->fd->fd, sess->ip, sess->port, sxe->generic, sxe->specific);
 
             break;
-            
+
         case event_STREAM:
 
             if(s->req_to == NULL) {
@@ -224,6 +218,25 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
             sess->packet_count++;
             sess->c2s->packet_count++;
 
+            /* check rate limits */
+            if(sess->stanza_rate != NULL) {
+                if(rate_check(sess->stanza_rate) == 0) {
+
+                    /* inform the app if we haven't already */
+                    if(!sess->stanza_rate_log) {
+                        if(s->state >= state_STREAM && sess->resources != NULL)
+                            log_write(sess->c2s->log, LOG_NOTICE, "[%d] [%s] is being stanza rate limited", sess->fd->fd, jid_user(sess->resources->jid));
+                        else
+                            log_write(sess->c2s->log, LOG_NOTICE, "[%d] [%s, port=%d] is being stanza rate limited", sess->fd->fd, sess->ip, sess->port);
+
+                        sess->stanza_rate_log = 1;
+                    }
+                }
+
+                /* update rate limits */
+                rate_add(sess->stanza_rate, 1);
+            }
+
             nad = (nad_t) data;
 
             /* we only want (message|presence|iq) in jabber:client, everything else gets dropped */
@@ -258,7 +271,7 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
                     if (jid == NULL || jid_reset_components(jid, jid->node, jid->domain, resource_buf) == NULL) {
                         log_debug(ZONE, "invalid jid data");
                         sx_nad_write(sess->s, stanza_error(nad, 0, stanza_err_BAD_REQUEST));
-                        
+
                         return 0;
                     }
 
@@ -289,7 +302,7 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
                 log_write(sess->c2s->log, LOG_NOTICE, "[%d] bound: jid=%s", sess->s->tag, jid_full(bres->jid));
 
                 /* build a result packet, we'll send this back to the client after we have a session for them */
-                sess->result = nad_new(sess->s->nad_cache);
+                sess->result = nad_new();
 
                 ns = nad_add_namespace(sess->result, uri_CLIENT, NULL);
 
@@ -330,7 +343,7 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
                 if(elem < 0 || NAD_CDATA_L(nad, elem) == 0) {
                     log_debug(ZONE, "no/empty resource given to unbind");
                     sx_nad_write(sess->s, stanza_error(nad, 0, stanza_err_BAD_REQUEST));
-                        
+
                     return 0;
                 }
 
@@ -338,7 +351,7 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
                 if(stringprep_xmpp_resourceprep(resource_buf, 1024) != 0) {
                     log_debug(ZONE, "cannot resourceprep");
                     sx_nad_write(sess->s, stanza_error(nad, 0, stanza_err_BAD_REQUEST));
-                    
+
                     return 0;
                 }
 
@@ -350,12 +363,12 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
                 if(bres == NULL) {
                     log_debug(ZONE, "resource /%s not bound", resource_buf);
                     sx_nad_write(sess->s, stanza_error(nad, 0, stanza_err_ITEM_NOT_FOUND));
-                    
+
                     return 0;
                 }
 
                 /* build a result packet, we'll send this back to the client after we close a session for them */
-                sess->result = nad_new(sess->s->nad_cache);
+                sess->result = nad_new();
 
                 ns = nad_add_namespace(sess->result, uri_CLIENT, NULL);
 
@@ -394,7 +407,7 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
                 log_debug(ZONE, "pre STARTTLS packet, dropping");
                 log_write(sess->c2s->log, LOG_NOTICE, "[%d] got pre STARTTLS packet, dropping", sess->s->tag);
 
-                sx_error(s, stream_err_NOT_AUTHORIZED, "stanza sent before starttls");
+                sx_error(s, stream_err_POLICY_VIOLATION, "STARTTLS is required for this stream");
 
                 nad_free(nad);
                 return 0;
@@ -432,9 +445,9 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
                     } else {
                         log_debug(ZONE, "packet without 'from' on multiple resource stream");
                     }
-    
+
                     sx_nad_write(sess->s, stanza_error(nad, 0, stanza_err_UNKNOWN_SENDER));
-    
+
                     return 0;
                 }
             } else
@@ -444,7 +457,7 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
             sm_packet(sess, bres, nad);
 
             break;
-        
+
         case event_OPEN:
 
             /* only send a result and bring us online if this wasn't a sasl auth */
@@ -459,7 +472,7 @@ static int _c2s_client_sx_callback(sx_t s, sx_event_t e, void *data, void *arg)
 
             /* they sasl auth'd, so we only want the new-style session start */
             else {
-                log_write(sess->c2s->log, LOG_NOTICE, "[%d] SASL authentication succeeded: mechanism=%s; authzid=%s%s", sess->s->tag, &sess->s->auth_method[5], sess->s->auth_id, sess->s->ssf ? ", TLS negotiated" : "");
+                log_write(sess->c2s->log, LOG_NOTICE, "[%d] SASL authentication succeeded: mechanism=%s; authzid=%s%s%s", sess->s->tag, &sess->s->auth_method[5], sess->s->auth_id, sess->s->ssf ? ", TLS negotiated" : "", sess->s->compressed ? ", ZLIB compression enabled" : "");
                 sess->sasl_authd = 1;
             }
 
@@ -490,7 +503,7 @@ static int _c2s_client_accept_check(c2s_t c2s, mio_fd_t fd, char *ip) {
         }
 
         if(rate_check(rt) == 0) {
-            log_write(c2s->log, LOG_NOTICE, "[%d] [%s] is being rate limited", fd->fd, ip);
+            log_write(c2s->log, LOG_NOTICE, "[%d] [%s] is being connect rate limited", fd->fd, ip);
             return 1;
         }
 
@@ -577,6 +590,12 @@ static int _c2s_client_mio_callback(mio_t m, mio_action_t a, mio_fd_t fd, void *
             if(c2s->byte_rate_total != 0)
                 sess->rate = rate_new(c2s->byte_rate_total, c2s->byte_rate_seconds, c2s->byte_rate_wait);
 
+            if(c2s->stanza_rate_total != 0)
+                sess->stanza_rate = rate_new(c2s->stanza_rate_total, c2s->stanza_rate_seconds, c2s->stanza_rate_wait);
+
+            /* give IP to SX */
+            sess->s->ip = sess->ip;
+
             /* find out which port this is */
             getsockname(fd->fd, (struct sockaddr *) &sa, &namelen);
             port = j_inet_getport(&sa);
@@ -650,7 +669,7 @@ static void _c2s_component_presence(c2s_t c2s, nad_t nad) {
                     sx_close(sess->s);
                 }
             } while(xhash_iter_next(c2s->sessions));
-        
+
         xhash_zap(c2s->sm_avail, from);
     }
 }
@@ -691,7 +710,7 @@ int c2s_router_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
                 log_write(c2s->log, LOG_NOTICE, "[%d] [router] read error: %s (%d)", c2s->fd->fd, MIO_STRERROR(MIO_ERROR), MIO_ERROR);
 
                 sx_kill(s);
-                
+
                 return -1;
             }
 
@@ -721,9 +740,9 @@ int c2s_router_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
                 return 0;
 
             log_write(c2s->log, LOG_NOTICE, "[%d] [router] write error: %s (%d)", c2s->fd->fd, MIO_STRERROR(MIO_ERROR), MIO_ERROR);
-        
+
             sx_kill(s);
-        
+
             return -1;
 
         case event_ERROR:
@@ -744,7 +763,7 @@ int c2s_router_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
             /* reset connection attempts counter */
             c2s->retry_left = c2s->retry_init;
 
-            nad = nad_new(c2s->router->nad_cache);
+            nad = nad_new();
             ns = nad_add_namespace(nad, uri_COMPONENT, NULL);
             nad_append_elem(nad, ns, "bind", 0);
             nad_append_attr(nad, -1, "name", c2s->id);
@@ -830,7 +849,7 @@ int c2s_router_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
                             log_write(c2s->log, LOG_NOTICE, "[%s, port=%d] listening for connections", c2s->local_ip, c2s->local_port);
                     } else
                         c2s->server_fd = NULL;
-            
+
 #ifdef HAVE_SSL
                     if(c2s->local_ssl_port != 0 && c2s->local_pemfile != NULL) {
                         c2s->server_ssl_fd = mio_listen(c2s->mio, c2s->local_ssl_port, c2s->local_ip, _c2s_client_mio_callback, (void *) c2s);
@@ -852,7 +871,7 @@ int c2s_router_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
 #endif
                     exit(1);
                 }
-            
+
                 /* we're online */
                 c2s->online = c2s->started = 1;
                 log_write(c2s->log, LOG_NOTICE, "ready for connections", c2s->id);
@@ -914,9 +933,53 @@ int c2s_router_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
             /* find the session, quietly drop if we don't have it */
             sess = xhash_get(c2s->sessions, skey);
             if(sess == NULL) {
-                /* !!! might want to send a stop to the sm; maybe it thinks we're still here? */
+                /* if we get this, the SM probably thinks the session is still active
+                 * so we need to tell SM to free it up */
                 log_debug(ZONE, "no session for %s", skey);
 
+                /* check if it's a started action; otherwise we could end up in an infinite loop
+                 * trying to tell SM to close in response to errors */
+                action = nad_find_attr(nad, 1, -1, "action", NULL);
+                if(action >= 0 && NAD_AVAL_L(nad, action) == 7 && strncmp("started", NAD_AVAL(nad, action), 7) == 0) {
+                    log_write(c2s->log, LOG_NOTICE, "session %s does not exist; telling sm to close", skey);
+
+                    /* we don't have a session and we don't have a resource; we need to forge them both
+                     * to get SM to close stuff */
+                    int target = nad_find_attr(nad, 1, -1, "target", NULL);
+                    smid = nad_find_attr(nad, 1, ns, "sm", NULL);
+                    if(target < 0 || smid < 0) {
+                        char *buf;
+                        int len;
+                        nad_print(nad, 0, &buf, &len);
+                        log_write(c2s->log, LOG_NOTICE, "sm sent an invalid start packet: %.*s", len, buf );
+                        nad_free(nad);
+                        return 0;
+                    }
+
+                    /* build temporary resource to close session for */
+                    bres_t tres = NULL;
+                    tres = (bres_t) calloc(1, sizeof(struct bres_st));
+                    tres->jid = jid_new(NAD_AVAL(nad, target), NAD_AVAL_L(nad, target));
+
+                    strncpy(tres->c2s_id, skey, sizeof(tres->c2s_id));
+                    snprintf(tres->sm_id, sizeof(tres->sm_id), "%.*s", NAD_AVAL_L(nad, smid), NAD_AVAL(nad, smid));
+
+                    /* make a temporary session */
+                    sess_t tsess = (sess_t) calloc(1, sizeof(struct sess_st));
+                    tsess->c2s = c2s;
+                    tsess->result = nad_new();
+                    strncpy(tsess->skey, skey, sizeof(tsess->skey));
+
+                    /* end a session with the sm */
+                    sm_end(tsess, tres);
+
+                    /* free our temporary messes */
+                    nad_free(tsess->result);
+                    jid_free(tres->jid); //TODO will this crash?
+                    free(tsess);
+                    free(tres);
+                }
+
                 nad_free(nad);
                 return 0;
             }
@@ -1034,7 +1097,7 @@ int c2s_router_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
                     /* no more resources bound? */
                     if(sess->bound < 1){
                         sess->active = 0;
-                    
+
                         /* return the unbind result to the client */
                         if(sess->result != NULL) {
                             sx_nad_write(sess->s, sess->result);
@@ -1043,10 +1106,10 @@ int c2s_router_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
 
                         if(replaced)
                             sx_error(sess->s, stream_err_CONFLICT, NULL);
-                        
+
                         /* close them */
                         sx_close(sess->s);
-    
+
                         nad_free(nad);
                         return 0;
                     }
@@ -1121,7 +1184,7 @@ int c2s_router_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
                             assert(ires != NULL);
                             ires->next = bres->next;
                         }
-    
+
                         jid_free(bres->jid);
                         free(bres);
 
