@@ -156,6 +156,17 @@ static void _c2s_config_expand(c2s_t c2s)
         }
     }
 
+    elem = config_get(c2s->config, "io.limits.stanzas");
+    if(elem != NULL)
+    {
+        c2s->stanza_rate_total = j_atoi(elem->values[0], 0);
+        if(c2s->stanza_rate_total != 0)
+        {
+            c2s->stanza_rate_seconds = j_atoi(j_attr((const char **) elem->attrs[0], "seconds"), 1);
+            c2s->stanza_rate_wait = j_atoi(j_attr((const char **) elem->attrs[0], "throttle"), 5);
+        }
+    }
+
     elem = config_get(c2s->config, "io.limits.connects");
     if(elem != NULL)
     {
@@ -169,6 +180,15 @@ static void _c2s_config_expand(c2s_t c2s)
 
     c2s->stanza_size_limit = j_atoi(config_get_one(c2s->config, "io.limits.stanzasize", 0), 0);
 
+    /* tweak timed checks with rate times */
+    if(c2s->io_check_interval == 0) {
+        if(c2s->byte_rate_total != 0)
+            c2s->io_check_interval = c2s->byte_rate_wait;
+
+        if(c2s->stanza_rate_total != 0 && c2s->io_check_interval > c2s->stanza_rate_wait)
+            c2s->io_check_interval = c2s->stanza_rate_wait;
+    }
+
     str = config_get_one(c2s->config, "io.access.order", 0);
     if(str == NULL || strcmp(str, "deny,allow") != 0)
         c2s->access = access_new(0);
@@ -305,7 +325,7 @@ static void _c2s_hosts_expand(c2s_t c2s)
 static int _c2s_router_connect(c2s_t c2s) {
     log_write(c2s->log, LOG_NOTICE, "attempting connection to router at %s, port=%d", c2s->router_ip, c2s->router_port);
 
-    c2s->fd = mio_connect(c2s->mio, c2s->router_port, c2s->router_ip, c2s_router_mio_callback, (void *) c2s);
+    c2s->fd = mio_connect(c2s->mio, c2s->router_port, c2s->router_ip, NULL, c2s_router_mio_callback, (void *) c2s);
     if(c2s->fd == NULL) {
         if(errno == ECONNREFUSED)
             c2s_lost_router = 1;
@@ -334,7 +354,7 @@ static int _c2s_sx_sasl_callback(int cb, void *arg, void **res, sx_t s, void *cb
 
     switch(cb) {
         case sx_sasl_cb_GET_REALM:
-            
+
             if(s->req_to == NULL)   /* this shouldn't happen */
                 my_realm = "";
 
@@ -394,7 +414,7 @@ static int _c2s_sx_sasl_callback(int cb, void *arg, void **res, sx_t s, void *cb
 
             return sx_sasl_ret_FAIL;
             break;
-        
+
         case sx_sasl_cb_CHECK_AUTHZID:
             creds = (sx_sasl_creds_t) arg;
 
@@ -463,7 +483,7 @@ static int _c2s_sx_sasl_callback(int cb, void *arg, void **res, sx_t s, void *cb
                     return sx_sasl_ret_FAIL;
             }
 
-            /* Using SSF is potentially dangerous, as SASL can alse set the
+            /* Using SSF is potentially dangerous, as SASL can also set the
              * SSF of the connection. However, SASL shouldn't do so until after
              * we've finished mechanism establishment 
              */
@@ -474,7 +494,7 @@ static int _c2s_sx_sasl_callback(int cb, void *arg, void **res, sx_t s, void *cb
                 if(config_get(c2s->config,buf) != NULL)
                     return sx_sasl_ret_OK;
             }
-            
+
             r = snprintf(buf, sizeof(buf), "authreg.mechanisms.sasl.%s",mechbuf);
             if (r < -1 || r > sizeof(buf))
                 return sx_sasl_ret_FAIL;
@@ -517,6 +537,13 @@ static void _c2s_time_checks(c2s_t c2s) {
                 sx_raw_write(sess->s, " ", 1);
             }
 
+            if(sess->rate != NULL && sess->rate->bad != 0 && rate_check(sess->rate) != 0) {
+                /* read the pending bytes when rate limit is no longer in effect */
+                log_debug(ZONE, "reading throttled %d", sess->fd->fd);
+                sess->s->want_read = 1;
+                sx_can_read(sess->s);
+            }
+
         } while(xhash_iter_next(c2s->sessions));
 }
 
@@ -525,11 +552,12 @@ JABBER_MAIN("jabberd2c2s", "Jabber 2 C2S", "Jabber Open Source Server: Client to
     c2s_t c2s;
     char *config_file;
     int optchar;
+    int mio_timeout;
     sess_t sess;
     bres_t res;
     union xhashv xhv;
     time_t check_time = 0;
-    
+
 #ifdef HAVE_UMASK
     umask((mode_t) 0027);
 #endif
@@ -542,9 +570,9 @@ JABBER_MAIN("jabberd2c2s", "Jabber 2 C2S", "Jabber Open Source Server: Client to
         WORD wVersionRequested;
         WSADATA wsaData;
         int err;
-        
+
         wVersionRequested = MAKEWORD( 2, 2 );
-        
+
         err = WSAStartup( wVersionRequested, &wsaData );
         if ( err != 0 ) {
             /* !!! tell user that we couldn't find a usable winsock dll */
@@ -662,13 +690,6 @@ JABBER_MAIN("jabberd2c2s", "Jabber 2 C2S", "Jabber Open Source Server: Client to
         }
     }
 #endif
-            
-    /* get sasl online */
-    c2s->sx_sasl = sx_env_plugin(c2s->sx_env, sx_sasl_init, "xmpp", _c2s_sx_sasl_callback, (void *) c2s);
-    if(c2s->sx_sasl == NULL) {
-        log_write(c2s->log, LOG_ERR, "failed to initialise SASL context, aborting");
-        exit(1);
-    }
 
 #ifdef HAVE_LIBZ
     /* get compression up and running */
@@ -679,8 +700,18 @@ JABBER_MAIN("jabberd2c2s", "Jabber 2 C2S", "Jabber Open Source Server: Client to
 #ifdef ENABLE_EXPERIMENTAL
     /* get stanza ack up */
     sx_env_plugin(c2s->sx_env, sx_ack_init);
+
+    /* and user IP address plugin */
+    sx_env_plugin(c2s->sx_env, sx_address_init);
 #endif
 
+    /* get sasl online */
+    c2s->sx_sasl = sx_env_plugin(c2s->sx_env, sx_sasl_init, "xmpp", _c2s_sx_sasl_callback, (void *) c2s);
+    if(c2s->sx_sasl == NULL) {
+        log_write(c2s->log, LOG_ERR, "failed to initialise SASL context, aborting");
+        exit(1);
+    }
+
     /* get bind up */
     sx_env_plugin(c2s->sx_env, bind_init, c2s);
 
@@ -694,8 +725,11 @@ JABBER_MAIN("jabberd2c2s", "Jabber 2 C2S", "Jabber Open Source Server: Client to
     c2s->retry_left = c2s->retry_init;
     _c2s_router_connect(c2s);
 
+    mio_timeout = ((c2s->io_check_interval != 0 && c2s->io_check_interval < 5) ?
+        c2s->io_check_interval : 5);
+
     while(!c2s_shutdown) {
-        mio_run(c2s->mio, 5);
+        mio_run(c2s->mio, mio_timeout);
 
         if(c2s_logrotate) {
             log_write(c2s->log, LOG_NOTICE, "reopening log ...");
@@ -726,7 +760,7 @@ JABBER_MAIN("jabberd2c2s", "Jabber 2 C2S", "Jabber Open Source Server: Client to
                 _c2s_router_connect(c2s);
             }
         }
-            
+
         /* cleanup dead sess (before sx_t as sess->result uses sx_t nad cache) */
         while(jqueue_size(c2s->dead_sess) > 0) {
             sess = (sess_t) jqueue_pull(c2s->dead_sess);
@@ -742,6 +776,7 @@ JABBER_MAIN("jabberd2c2s", "Jabber 2 C2S", "Jabber Open Source Server: Client to
                     res = tmp;
                 }
             if(sess->rate != NULL) rate_free(sess->rate);
+            if(sess->stanza_rate != NULL) rate_free(sess->stanza_rate);
 
             free(sess);
         }
@@ -776,13 +811,13 @@ JABBER_MAIN("jabberd2c2s", "Jabber 2 C2S", "Jabber Open Source Server: Client to
                     c2s_shutdown = 1;
                 }
             }
-    
+
             check_time = time(NULL);
         }
     }
 
     log_write(c2s->log, LOG_NOTICE, "shutting down");
-    
+
     if(xhash_iter_first(c2s->sessions))
         do {
             xhv.sess_val = &sess;
