@@ -258,7 +258,7 @@ static int _in_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
         case event_PACKET:
             /* we're counting packets */
             in->packet_count++;
-	    in->s2s->packet_count++;
+            in->s2s->packet_count++;
 
             nad = (nad_t) data;
 
@@ -396,7 +396,7 @@ static void _in_result(conn_t in, nad_t nad) {
     free(rkey);
 
     /* new packet */
-    verify = nad_new(in->s2s->router->nad_cache);
+    verify = nad_new();
     ns = nad_add_namespace(verify, uri_DIALBACK, "db");
 
     nad_append_elem(verify, ns, "verify", 0);
@@ -548,14 +548,21 @@ static void _in_packet(conn_t in, nad_t nad) {
         }
     }
 
-    ns = nad_find_namespace(nad, 0, uri_CLIENT, NULL);
-    if(ns < 0) {
+    /*
+     * If stanza is not in any namespace (either because we removed the
+     * jabber:server namespace above or because it's in the default
+     * namespace for this stream) then this packet is intended to be
+     * handled by sm (and not just routed through the server), so set the
+     * jabber:client namespace.
+     */
+    if(ns >= 0 || nad->elems[0].ns < 0) {
         ns = nad_add_namespace(nad, uri_CLIENT, NULL);
         nad->scope = -1;
         nad->nss[ns].next = nad->elems[0].ns;
         nad->elems[0].ns = ns;
     }
-    nad->elems[0].my_ns = ns;
+
+    nad->elems[0].my_ns = nad->elems[0].ns;
 
     /* wrap up the packet */
     ns = nad_add_namespace(nad, uri_COMPONENT, "comp");
