@@ -115,7 +115,15 @@ static void _s2s_config_expand(s2s_t s2s) {
 
     s2s->packet_stats = config_get_one(s2s->config, "stats.packet", 0);
 
+    /*
+    * If no origin IP is specified, use local IP as the originating one:
+     * it makes most sense, at least for SSL'ized connections.
+     */
     s2s->local_ip = config_get_one(s2s->config, "local.ip", 0);
+    s2s->origin_ip = config_get_one(s2s->config, "local.origin", 0);
+    if(s2s->origin_ip == NULL && s2s->local_ip != NULL)
+        s2s->origin_ip = s2s->local_ip;
+
     if(s2s->local_ip == NULL)
         s2s->local_ip = "0.0.0.0";
 
@@ -163,6 +171,7 @@ static void _s2s_config_expand(s2s_t s2s) {
     if (s2s->dns_min_ttl < 5)
         s2s->dns_min_ttl = 5;
     s2s->dns_max_ttl = j_atoi(config_get_one(s2s->config, "lookup.max-ttl", 0), 86400);
+    s2s->etc_hosts_ttl = j_atoi(config_get_one(s2s->config, "lookup.etc-hosts-ttl", 0), 86400);
     s2s->out_reuse = config_count(s2s->config, "out-conn-reuse") ? 1 : 0;
 }
 
@@ -227,7 +236,7 @@ static void _s2s_hosts_expand(s2s_t s2s)
 static int _s2s_router_connect(s2s_t s2s) {
     log_write(s2s->log, LOG_NOTICE, "attempting connection to router at %s, port=%d", s2s->router_ip, s2s->router_port);
 
-    s2s->fd = mio_connect(s2s->mio, s2s->router_port, s2s->router_ip, s2s_router_mio_callback, (void *) s2s);
+    s2s->fd = mio_connect(s2s->mio, s2s->router_port, s2s->router_ip, NULL, s2s_router_mio_callback, (void *) s2s);
     if(s2s->fd == NULL) {
         if(errno == ECONNREFUSED)
             s2s_lost_router = 1;
