@@ -20,6 +20,8 @@
 
 #include "s2s.h"
 
+#include <idna.h>
+
 /*
  * we handle packets going from the router to the world, and stuff
  * that comes in on connections we initiated.
@@ -137,7 +139,7 @@ static void _out_dialback(conn_t out, char *rkey) {
     /* kick off the dialback */
     dbkey = s2s_db_key(NULL, out->s2s->local_secret, c, out->s->id);
 
-    nad = nad_new(out->s->nad_cache);
+    nad = nad_new();
 
     /* request auth */
     ns = nad_add_namespace(nad, uri_DIALBACK, "db");
@@ -501,7 +503,7 @@ int out_route(s2s_t s2s, char *route, conn_t *out, int allow_bad) {
             /* connect */
             log_debug(ZONE, "initiating connection to %s", ipport);
 
-            (*out)->fd = mio_connect(s2s->mio, port, ip, _out_mio_callback, (void *) *out);
+            (*out)->fd = mio_connect(s2s->mio, port, ip, s2s->origin_ip, _out_mio_callback, (void *) *out);
 
             if ((*out)->fd == NULL) {
                 dnsres_t bad;
@@ -571,7 +573,7 @@ int out_route(s2s_t s2s, char *route, conn_t *out, int allow_bad) {
 }
 
 /** send a packet out */
-void out_packet(s2s_t s2s, pkt_t pkt) {
+int out_packet(s2s_t s2s, pkt_t pkt) {
     char *rkey;
     conn_t out;
     conn_state_t state;
@@ -591,10 +593,13 @@ void out_packet(s2s_t s2s, pkt_t pkt) {
         if (ret) {
             /* bounce queue */
             out_bounce_route_queue(s2s, rkey, stanza_err_SERVICE_UNAVAILABLE);
+
+            free(rkey);
+            return -1;
         }
 
         free(rkey);
-        return;
+        return 0;
     }
 
     /* connection in progress */
@@ -604,7 +609,7 @@ void out_packet(s2s_t s2s, pkt_t pkt) {
         _out_packet_queue(s2s, pkt);
 
         free(rkey);
-        return;
+        return 0;
     }
 
     /* connection state */
@@ -647,7 +652,7 @@ void out_packet(s2s_t s2s, pkt_t pkt) {
         free(pkt);
 
         free(rkey);
-        return;
+        return 0;
     }
 
     /* can't be handled yet, queue */
@@ -656,13 +661,14 @@ void out_packet(s2s_t s2s, pkt_t pkt) {
     /* if dialback is in progress, then we're done for now */
     if(state == conn_INPROGRESS) {
         free(rkey);
-        return;
+        return 0;
     }
 
     /* this is a new route - send dialback auth request to piggyback on the existing connection */
     _out_dialback(out, rkey);
 
     free(rkey);
+    return 0;
 }
 
 char *dns_make_ipport(char *host, int port) {
@@ -903,6 +909,82 @@ static void _dns_result_aaaa(struct dns_ctx *ctx, struct dns_rr_a6 *result, void
     free(result);
 }
 
+/* try /etc/hosts if the A process did not return any results */
+static int _etc_hosts_lookup(const char *cszName, char *szIP, const int ciMaxIPLen) {
+#define EHL_LINE_LEN 260
+    int iSuccess = 0;
+    size_t iLen;
+    char szLine[EHL_LINE_LEN + 1]; /* one extra for the space character (*) */
+    char *pcStart, *pcEnd;
+    FILE *fHosts;
+
+    do {
+        /* initialization */
+        fHosts = NULL;
+
+        /* sanity checks */
+        if ((cszName == NULL) || (szIP == NULL) || (ciMaxIPLen <= 0))
+            break;
+        szIP[0] = 0;
+
+        /* open the hosts file */
+#ifdef _WIN32
+        pcStart = getenv("WINDIR");
+        if (pcStart != NULL) {
+            sprintf(szLine, "%s\\system32\\drivers\\etc\\hosts", pcStart);
+        } else {
+            strcpy(szLine, "C:\\WINDOWS\\system32\\drivers\\etc\\hosts");
+        }
+#else
+        strcpy(szLine, "/etc/hosts");
+#endif
+        fHosts = fopen(szLine, "r");
+        if (fHosts == NULL)
+            break;
+
+        /* read line by line ... */
+        while (fgets(szLine, EHL_LINE_LEN, fHosts) != NULL) {
+            /* remove comments */
+            pcStart = strchr (szLine, '#');
+            if (pcStart != NULL)
+                *pcStart = 0;
+            strcat(szLine, " "); /* append a space character for easier parsing (*) */
+
+            /* first to appear: IP address */
+            iLen = strspn(szLine, "1234567890.");
+            if ((iLen < 7) || (iLen > 15)) /* superficial test for anything between x.x.x.x and xxx.xxx.xxx.xxx */
+                continue;
+            pcEnd = szLine + iLen;
+            *pcEnd = 0;
+            pcEnd++; /* not beyond the end of the line yet (*) */
+
+            /* check strings separated by blanks, tabs or newlines */
+            pcStart = pcEnd + strspn(pcEnd, " \t\n");
+            while (*pcStart != 0) {
+                pcEnd = pcStart + strcspn(pcStart, " \t\n");
+                *pcEnd = 0;
+                pcEnd++; /* not beyond the end of the line yet (*) */
+
+                if (strcasecmp(pcStart, cszName) == 0) {
+                    strncpy(szIP, szLine, ciMaxIPLen - 1);
+                    szIP[ciMaxIPLen - 1] = 0;
+                    iSuccess = 1;
+                    break;
+                }
+
+                pcStart = pcEnd + strspn(pcEnd, " \t\n");
+            }
+            if (iSuccess)
+                break;
+        }
+    } while (0);
+
+    if (fHosts != NULL)
+        fclose(fHosts);
+
+    return (iSuccess);
+}
+
 /* this function is called with a NULL ctx to start the A/AAAA process */
 static void _dns_result_a(struct dns_ctx *ctx, struct dns_rr_a4 *result, void *data) {
     dnsquery_t query = data;
@@ -910,8 +992,18 @@ static void _dns_result_a(struct dns_ctx *ctx, struct dns_rr_a4 *result, void *d
     query->query = NULL;
 
     if (ctx != NULL && result == NULL) {
-        log_debug(ZONE, "dns failure for %s@%p: A %s (%d)", query->name, query,
-            query->cur_host, dns_status(ctx));
+#define DRA_IP_LEN 16
+        char szIP[DRA_IP_LEN];
+        if (_etc_hosts_lookup (query->name, szIP, DRA_IP_LEN)) {
+            log_debug(ZONE, "/etc/lookup for %s@%p: %s (%d)", query->name,
+                query, szIP, query->s2s->etc_hosts_ttl);
+
+            _dns_add_result (query, szIP, query->cur_port,
+                query->cur_prio, query->cur_weight, query->s2s->etc_hosts_ttl);
+        } else {
+            log_debug(ZONE, "dns failure for %s@%p: A %s (%d)", query->name, query,
+                query->cur_host, dns_status(ctx));
+        }
     } else if (result != NULL) {
         char ip[INET_ADDRSTRLEN];
         int i;
@@ -982,12 +1074,13 @@ static void _dns_result_a(struct dns_ctx *ctx, struct dns_rr_a4 *result, void *d
                 _dns_result_a(ctx, NULL, query);
         }
 
-        *c = '/';
         c--;
+        *c = '/';
 
     /* finished */
     } else {
         time_t now = time(NULL);
+        char *domain;
 
         free(query->cur_host);
         query->cur_host = NULL;
@@ -1026,7 +1119,14 @@ static void _dns_result_a(struct dns_ctx *ctx, struct dns_rr_a4 *result, void *d
 
         xhash_free(query->hosts);
         query->hosts = NULL;
-        out_resolve(query->s2s, query->name, query->results, query->expiry);
+        if (idna_to_unicode_8z8z(query->name, &domain, 0) != IDNA_SUCCESS)
+        {
+            log_write(query->s2s->log, LOG_ERR, "idna dns decode for %s failed", query->name);
+            /* TODO: Is it better to shortcut resolution failure here? */
+            domain = strdup(query->name);
+        }
+        out_resolve(query->s2s, domain, query->results, query->expiry);
+        free(domain);
         free(query->name);
         free(query);
     }
@@ -1036,7 +1136,12 @@ void dns_resolve_domain(s2s_t s2s, dnscache_t dns) {
     dnsquery_t query = (dnsquery_t) calloc(1, sizeof(struct dnsquery_st));
 
     query->s2s = s2s;
-    query->name = strdup(dns->name);
+    if (idna_to_ascii_8z(dns->name, &query->name, 0) != IDNA_SUCCESS)
+    {
+        log_write(s2s->log, LOG_ERR, "idna dns encode for %s failed", dns->name);
+        /* TODO: Is it better to shortcut resolution failure here? */
+        query->name = strdup(dns->name);
+    }
     query->srv_i = -1;
     query->hosts = xhash_new(71);
     query->results = xhash_new(71);
@@ -1357,7 +1462,17 @@ static int _out_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
             sxe = (sx_error_t *) data;
             log_write(out->s2s->log, LOG_NOTICE, "[%d] [%s, port=%d] error: %s (%s)", out->fd->fd, out->ip, out->port, sxe->generic, sxe->specific);
 
-            if (!out->online) {
+            /* mark as bad if we did not manage to connect or there is unrecoverable stream error */
+            if (!out->online ||
+                    (sxe->code == SX_ERR_STREAM &&
+                        (strstr(sxe->specific, "host-gone") ||        /* it's not there now */
+                         strstr(sxe->specific, "host-unknown") ||     /* they do not service the host */
+                         strstr(sxe->specific, "not-authorized") ||   /* they do not want us there */
+                         strstr(sxe->specific, "see-other-host") ||   /* we do not support redirections yet */
+                         strstr(sxe->specific, "system-shutdown") ||  /* they are going down */
+                         strstr(sxe->specific, "unsupported-version") /* they do not support our stream version */
+                        )))
+            {
                 dnsres_t bad;
                 char *ipport;
 
@@ -1426,7 +1541,7 @@ static int _out_sx_callback(sx_t s, sx_event_t e, void *data, void *arg) {
                         elem = nad_find_elem(nad, 0, ns, "starttls", 1);
                         if(elem >= 0) {
                             log_debug(ZONE, "got STARTTLS in stream features");
-                            if(sx_ssl_client_starttls(out->s2s->sx_ssl, s, NULL) == 0) {
+                            if(sx_ssl_client_starttls(out->s2s->sx_ssl, s, out->s2s->local_pemfile) == 0) {
                                 starttls = 1;
                                 nad_free(nad);
                                 return 0;
@@ -1507,7 +1622,7 @@ static void _out_result(conn_t out, nad_t nad) {
 
     /* key is valid */
     if(nad_find_attr(nad, 0, -1, "type", "valid") >= 0) {
-        log_write(out->s2s->log, LOG_NOTICE, "[%d] [%s, port=%d] outgoing route '%s' is now valid%s", out->fd->fd, out->ip, out->port, rkey, out->s->ssf ? ", TLS negotiated" : "");
+        log_write(out->s2s->log, LOG_NOTICE, "[%d] [%s, port=%d] outgoing route '%s' is now valid%s", out->fd->fd, out->ip, out->port, rkey, (out->s->flags & SX_SSL_WRAPPER) ? ", TLS negotiated" : "");
 
         xhash_put(out->states, pstrdup(xhash_pool(out->states), rkey), (void *) conn_VALID);    /* !!! small leak here */
 
@@ -1596,7 +1711,7 @@ static void _out_verify(conn_t out, nad_t nad) {
     attr = nad_find_attr(nad, 0, -1, "type", "valid");
     if(attr >= 0) {
         xhash_put(in->states, pstrdup(xhash_pool(in->states), rkey), (void *) conn_VALID);
-        log_write(in->s2s->log, LOG_NOTICE, "[%d] [%s, port=%d] incoming route '%s' is now valid%s", in->fd->fd, in->ip, in->port, rkey, in->s->ssf ? ", TLS negotiated" : "");
+        log_write(in->s2s->log, LOG_NOTICE, "[%d] [%s, port=%d] incoming route '%s' is now valid%s", in->fd->fd, in->ip, in->port, rkey, (in->s->flags & SX_SSL_WRAPPER) ? ", TLS negotiated" : "");
         valid = 1;
     } else {
         log_write(in->s2s->log, LOG_NOTICE, "[%d] [%s, port=%d] incoming route '%s' is now invalid", in->fd->fd, in->ip, in->port, rkey);
@@ -1611,7 +1726,7 @@ static void _out_verify(conn_t out, nad_t nad) {
     --out->verify;
 
     /* let them know what happened */
-    nad = nad_new(in->s->nad_cache);
+    nad = nad_new();
 
     ns = nad_add_namespace(nad, uri_DIALBACK, "db");
     nad_append_elem(nad, ns, "result", 0);
@@ -1644,7 +1759,8 @@ int out_bounce_domain_queues(s2s_t s2s, const char *domain, int err)
   if (xhash_iter_first(s2s->outq)) {
       do {
           xhash_iter_get(s2s->outq, (const char **) &rkey, NULL);
-          pktcount += out_bounce_route_queue(s2s, rkey, err);
+          if(s2s_route_key_match(NULL, domain, rkey))
+              pktcount += out_bounce_route_queue(s2s, rkey, err);
       } while(xhash_iter_next(s2s->outq));
   }
 
@@ -1720,7 +1836,7 @@ void out_flush_domain_queues(s2s_t s2s, const char *domain) {
 void out_flush_route_queue(s2s_t s2s, char *rkey) {
     jqueue_t q;
     pkt_t pkt;
-    int npkt, i;
+    int npkt, i, ret;
 
     q = xhash_get(s2s->outq, rkey);
     if(q == NULL)
@@ -1731,8 +1847,15 @@ void out_flush_route_queue(s2s_t s2s, char *rkey) {
 
     for(i = 0; i < npkt; i++) {
         pkt = jqueue_pull(q);
-        if(pkt)
-            out_packet(s2s, pkt);
+        if(pkt) {
+            ret = out_packet(s2s, pkt);
+            if (ret) {
+                /* uh-oh. the queue was deleted...
+                   q and pkt have been freed
+                   if q->key == rkey, rkey has also been freed */
+                return;
+            }
+        }
     }
 
     /* delete queue for route and remove route from queue hash */
