@@ -30,6 +30,7 @@
 
 #include "c2s.h"
 #include <lber.h>
+#define LDAP_DEPRECATED 1
 #include <ldap.h>
 
 #define AR_LDAP_FLAGS_NONE      (0x0)
@@ -430,7 +431,7 @@ int ar_init(authreg_t ar)
     basedn = config_get(ar->c2s->config, ldap_entry);
     if(basedn == NULL)
     {
-        log_write(ar->c2s->log, LOG_ERR, "ldap: no basedns specified in config file");
+        log_write(ar->c2s->log, LOG_ERR, "ldap: no basedn specified in config file");
         return 1;
     }
 
@@ -482,9 +483,9 @@ int ar_init(authreg_t ar)
         data->flags |= AR_LDAP_FLAGS_V3;
 
     if (l>0)
-        snprintf(ldap_entry,sizeof(ldap_entry), "authreg.ldap%d.startls", l );
+        snprintf(ldap_entry,sizeof(ldap_entry), "authreg.ldap%d.starttls", l );
     else
-        snprintf(ldap_entry, sizeof(ldap_entry), "authreg.ldap.startls");
+        snprintf(ldap_entry, sizeof(ldap_entry), "authreg.ldap.starttls");
     if(config_get(ar->c2s->config, ldap_entry) != NULL)
         data->flags |= AR_LDAP_FLAGS_STARTTLS;
 
