@@ -34,6 +34,7 @@
 #ifdef STORAGE_LDAP
 
 #include <lber.h>
+#define LDAP_DEPRECATED 1
 #include <ldap.h>
 
 #define LDAPFULL_PASSBUF_MAX 257
@@ -703,7 +704,7 @@ DLLEXPORT int ar_init(authreg_t ar)
     basedn = config_get(ar->c2s->config, "authreg.ldapfull.basedn");
     if(basedn == NULL)
     {
-        log_write(ar->c2s->log, LOG_ERR, "ldap: no basedns specified in config file");
+        log_write(ar->c2s->log, LOG_ERR, "ldap: no basedn specified in config file");
         return 1;
     }
 
