@@ -651,7 +651,7 @@ static nad_t _sx_sasl_success(sx_t s) {
     nad_t nad;
     int ns;
 
-    nad = nad_new(s->nad_cache);
+    nad = nad_new();
     ns = nad_add_namespace(nad, uri_SASL, NULL);
 
     nad_append_elem(nad, ns, "success", 0);
@@ -664,7 +664,7 @@ static nad_t _sx_sasl_failure(sx_t s, const char *err) {
     nad_t nad;
     int ns;
 
-    nad = nad_new(s->nad_cache);
+    nad = nad_new();
     ns = nad_add_namespace(nad, uri_SASL, NULL);
 
     nad_append_elem(nad, ns, "failure", 0);
@@ -679,7 +679,7 @@ static nad_t _sx_sasl_challenge(sx_t s, char *data, int dlen) {
     nad_t nad;
     int ns;
 
-    nad = nad_new(s->nad_cache);
+    nad = nad_new();
     ns = nad_add_namespace(nad, uri_SASL, NULL);
 
     nad_append_elem(nad, ns, "challenge", 0);
@@ -694,7 +694,7 @@ static nad_t _sx_sasl_response(sx_t s, char *data, int dlen) {
     nad_t nad;
     int ns;
 
-    nad = nad_new(s->nad_cache);
+    nad = nad_new();
     ns = nad_add_namespace(nad, uri_SASL, NULL);
 
     nad_append_elem(nad, ns, "response", 0);
@@ -709,7 +709,7 @@ static nad_t _sx_sasl_abort(sx_t s) {
     nad_t nad;
     int ns;
 
-    nad = nad_new(s->nad_cache);
+    nad = nad_new();
     ns = nad_add_namespace(nad, uri_SASL, NULL);
 
     nad_append_elem(nad, ns, "abort", 0);
@@ -719,17 +719,14 @@ static nad_t _sx_sasl_abort(sx_t s) {
 
 /** utility: decode incoming handshake data */
 static void _sx_sasl_decode(char *in, int inlen, char **out, int *outlen) {
-    *outlen = apr_base64_decode_len(in, inlen);
-    *out = (char *) malloc(sizeof(char) * (*outlen + 1));
-    apr_base64_decode(*out, in, inlen);
+    *out = (char *) malloc(sizeof(char) * (2 * inlen));
+    sasl_decode64(in,inlen,out,2*inlen,outlen);
 }
 
 /** utility: encode outgoing handshake data */
 static void _sx_sasl_encode(char *in, int inlen, char **out, int *outlen) {
-    *outlen = apr_base64_encode_len(inlen);
-    *out = (char *) malloc(sizeof(char) * *outlen);
-    apr_base64_encode(*out, in, inlen);
-    (*outlen)--;
+    *out = (char *) malloc(sizeof(char) * (2 * inlen));
+    sasl_encode64(in,inlen,out,2*inlen,outlen);
 }
 
 /** auth done, restart the stream */
@@ -1288,7 +1285,7 @@ int sx_sasl_auth(sx_plugin_t p, sx_t s, char *appname, char *mech, char *user, c
     _sx_sasl_encode(out, outlen, &buf, &buflen);
 
     /* build the nad */
-    nad = nad_new(s->nad_cache);
+    nad = nad_new();
     ns = nad_add_namespace(nad, uri_SASL, NULL);
 
     nad_append_elem(nad, ns, "auth", 0);
