@@ -102,7 +102,7 @@ static int _sx_ssl_process(sx_t s, sx_plugin_t p, nad_t nad) {
     if(s->type == type_SERVER) {
         if(NAD_ENAME_L(nad, 0) == 8 && strncmp(NAD_ENAME(nad, 0), "starttls", 8) == 0) {
             nad_free(nad);
-    
+
             /* can't go on if we've been here before */
             if(s->ssf > 0) {
                 _sx_debug(ZONE, "starttls requested on already encrypted channel, dropping packet");
@@ -251,8 +251,8 @@ static int _sx_ssl_handshake(sx_t s, _sx_ssl_conn_t sc) {
                 _sx_gen_error(sxe, SX_ERR_SSL, "SSL handshake error", errstring);
                 _sx_event(s, event_ERROR, (void *) &sxe);
 
-                _sx_error(s, stream_err_INTERNAL_SERVER_ERROR, errstring);
-                _sx_close(s);
+                sx_error(s, stream_err_INTERNAL_SERVER_ERROR, errstring);
+                sx_close(s);
 
                 /* !!! drop queue */
 
@@ -271,9 +271,9 @@ static int _sx_ssl_wio(sx_t s, sx_plugin_t p, sx_buf_t buf) {
     char *errstring;
     sx_error_t sxe;
 
-    /* sanity */
+    /* do not encrypt when error */
     if(sc->last_state == SX_SSL_STATE_ERROR)
-        return -2;
+        return 1;
 
     _sx_debug(ZONE, "in _sx_ssl_wio");
 
@@ -332,8 +332,8 @@ static int _sx_ssl_wio(sx_t s, sx_plugin_t p, sx_buf_t buf) {
                 _sx_gen_error(sxe, SX_ERR_SSL, "SSL handshake error", errstring);
                 _sx_event(s, event_ERROR, (void *) &sxe);
 
-                _sx_error(s, stream_err_INTERNAL_SERVER_ERROR, errstring);
-                _sx_close(s);
+                sx_error(s, stream_err_INTERNAL_SERVER_ERROR, errstring);
+                sx_close(s);
 
                 /* !!! drop queue */
 
@@ -465,8 +465,8 @@ static int _sx_ssl_rio(sx_t s, sx_plugin_t p, sx_buf_t buf) {
                 _sx_gen_error(sxe, SX_ERR_SSL, "SSL handshake error", errstring);
                 _sx_event(s, event_ERROR, (void *) &sxe);
 
-                _sx_error(s, stream_err_INTERNAL_SERVER_ERROR, errstring);
-                _sx_close(s);
+                sx_error(s, stream_err_INTERNAL_SERVER_ERROR, errstring);
+                sx_close(s);
 
                 /* !!! drop queue */
 
@@ -484,7 +484,7 @@ static int _sx_ssl_rio(sx_t s, sx_plugin_t p, sx_buf_t buf) {
     /* flag if we want to read */
     if(sc->last_state == SX_SSL_STATE_WANT_READ || sc->last_state == SX_SSL_STATE_NONE)
         s->want_read = 1;
-    
+
     if(buf->len == 0)
         return 0;
 
@@ -524,6 +524,7 @@ static void _sx_ssl_client(sx_t s, sx_plugin_t p) {
     SSL_set_bio(sc->ssl, sc->rbio, sc->wbio);
     SSL_set_connect_state(sc->ssl);
     SSL_set_ssl_method(sc->ssl, TLSv1_client_method());
+    SSL_set_options(sc->ssl, SSL_OP_NO_TICKET);
 
     /* empty external_id */
     sc->external_id = NULL;
@@ -654,7 +655,7 @@ static void _sx_ssl_free(sx_t s, sx_plugin_t p) {
     }
 
     free(sc);
-    
+
     s->plugin_data[p->index] = NULL;
 }
 
@@ -738,27 +739,28 @@ int sx_ssl_server_addcert(sx_plugin_t p, char *name, char *pemfile, char *cachai
     if(pemfile == NULL)
         return 1;
 
-    /* !!! output openssl error messages to the debug log */
+    /* begin with fresh error stack */
+    ERR_clear_error();
 
     /* create the context */
     ctx = SSL_CTX_new(SSLv23_method());
     if(ctx == NULL) {
-        _sx_debug(ZONE, "ssl context creation failed");
+        _sx_debug(ZONE, "ssl context creation failed; %s", ERR_error_string(ERR_get_error(), NULL));
         return 1;
     }
-    
+
     /* Load the CA chain, if configured */
     if (cachain != NULL) {
         ret = SSL_CTX_load_verify_locations (ctx, cachain, NULL);
         if(ret != 1) {
-            _sx_debug(ZONE, "WARNING: couldn't load CA chain: %s", cachain);
+            _sx_debug(ZONE, "WARNING: couldn't load CA chain: %s; %s", cachain, ERR_error_string(ERR_get_error(), NULL));
         }
     }
 
     /* load the certificate */
     ret = SSL_CTX_use_certificate_chain_file(ctx, pemfile);
     if(ret != 1) {
-        _sx_debug(ZONE, "couldn't load certificate from %s", pemfile);
+        _sx_debug(ZONE, "couldn't load certificate from %s; %s", pemfile, ERR_error_string(ERR_get_error(), NULL));
         SSL_CTX_free(ctx);
         return 1;
     }
@@ -766,7 +768,7 @@ int sx_ssl_server_addcert(sx_plugin_t p, char *name, char *pemfile, char *cachai
     /* load the private key */
     ret = SSL_CTX_use_PrivateKey_file(ctx, pemfile, SSL_FILETYPE_PEM);
     if(ret != 1) {
-        _sx_debug(ZONE, "couldn't load private key from %s", pemfile);
+        _sx_debug(ZONE, "couldn't load private key from %s; %s", pemfile, ERR_error_string(ERR_get_error(), NULL));
         SSL_CTX_free(ctx);
         return 1;
     }
@@ -774,7 +776,7 @@ int sx_ssl_server_addcert(sx_plugin_t p, char *name, char *pemfile, char *cachai
     /* check the private key matches the certificate */
     ret = SSL_CTX_check_private_key(ctx);
     if(ret != 1) {
-        _sx_debug(ZONE, "private key does not match certificate public key");
+        _sx_debug(ZONE, "private key does not match certificate public key; %s", ERR_error_string(ERR_get_error(), NULL));
         SSL_CTX_free(ctx);
         return 1;
     }
